package Cuitter::Plugin::Preload;
use Mouse::Role;

requires qw( plugins output load_plugins cache set_help );

sub _init_Preload {
    my $self = shift;
    $self->set_help( 'preload', '(Plugin-Name) Load specified plugin. When starting after next time, the plugin loaded by this method comes to be load automatically.' );
    $self->set_help( 'unload', '(Plugin-Name) When being start next time, the specified plugin is not automatically loaded.' );
    $self->set_help( 'show_preload', 'Show loads plugin when starting.' );
    my $preload = $self->cache->get( 'preload' );
    if ( $preload ) {
        for my $plugin ( @{ $preload } ) {
            my $init = "_init_$plugin";
            $self->load_plugin( $plugin );
            $self->$init;
        }
    }
}

sub com_preload {
    my ( $self, $plugin ) = @_;
    return 1 unless $plugin;
    eval {
        $self->load_plugin( $plugin );
    };
    if ( $@ ) {
        $self->output->print( $@."\n" );
    }
    else {
        my $init = "_init_$plugin";
        $self->$init;
        my $preload = $self->cache->get( 'preload' );
        $preload ||= [];
        push @{ $preload }, $plugin;
        $self->cache->set( 'preload', $preload );
        $self->output->print( "Done.\n" );
    }
    return 1;
}

sub com_unload {
    my ( $self, $plugin ) = @_;
    return 1 unless $plugin;
    my $preload = $self->cache->get( 'preload' );
    if ( $preload ) {
        my @tmp_preload = ();
        for my $p ( @{ $preload } ) {
            push @tmp_preload, $p if $p ne $plugin;
        }
        $self->cache->set( 'preload', \@tmp_preload );
        $self->output->print( "Done\n" );
    }
    return 1;
}

sub com_show_preload {
    my $self = shift;
    my $preload = $self->cache->get( 'preload' );
    if ( $preload ) {
        $self->output->print( "Following plugins will be load when beginning\n" );
        for my $plugin ( @{ $preload } ) {
            $self->output->print( " - $plugin\n" );
        }
    }
    else {
        $self->output->print( "No plugins will be load when beginning\n" );
    }
    return 1;
}

no Mouse::Role;
1;
__END__

=head1 NAME

Cuitter::Plugin::Preload - a Cuitter Plugin

=head1 SYNOPSIS

This plugin will be load automatically when beginning.

=head1 DESCRIPTION

It's a Plugin Auto Loader.

=head1 Provides Command

=head2 /preload [ plugin-name ] 

Load specified plugin. When starting after next time, the plugin loaded by this method comes to be load automatically.

=head2 /unload [ plugin-name ]

When being start next time, the specified plugin is not automatically loaded.

=head2 /show_preload

Show loads plugin when starting.

=head1 AUTHOR

Satoshi Azuma E<lt>ytnobody@gmail.comE<gt>

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

